if select(6, GetAddOnInfo("PitBull_" .. (debugstack():match("[d%.][d%.][O%.]ns\\PitBull\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 34615 $"):match("%d+"))

local PitBull = PitBull
local PitBull_Experience = PitBull:NewModule("Experience", "AceEvent-2.0", "AceConsole-2.0")
local self = PitBull_Experience
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-05-06 05:32:36 -0400 (Sun, 06 May 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

-- localization placeholder
local L = PitBull:L("PitBull-Experience")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

local DogTag = AceLibrary("DogTag-1.0")

local repColor = {
	[1] = {r = 226/255, g = 45/255, b = 75/255}, -- Hated
	[2] = {r = 226/255, g = 45/255, b = 75/255}, -- Hostile
	[3] = {r = 0.75, g = 0.27, b = 0}, -- unfriendly
	[4] = {r = 1, g = 1, b = 34/255}, -- neutral
	[5] = {r = 0.2, g = 0.8, b = 0.15}, -- friendly
	[6] = {r = 0.2, g = 0.8, b = 0.15}, -- honored
	[7] = {r = 0.2, g = 0.8, b = 0.15}, -- revered
	[8] = {r = 0.2, g = 0.8, b = 0.15}, -- exalted
}

PitBull_Experience:RegisterPitBullChildFrames('expBar', 'expBarText')

local showPet = select(2, UnitClass("player")) == "HUNTER"

function PitBull_Experience:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("Experience")
	PitBull:RegisterDefaults("Experience", "profile", {
		['**'] = {
			ignore = false,
			deficit = false,
			reverse = false,
		},
		player = {
			ignore = true,
		},
		colors = {0, 0, 1},
		restColors = {1, 0, 1},
		bgColors = {0, 0, 0},
		textColor = {1, 1, 1},
	})
end

function PitBull_Experience:OnEnable(first)
	self:RegisterEvent("PLAYER_XP_UPDATE")
	self:RegisterEvent("UPDATE_EXHAUSTION", "PLAYER_XP_UPDATE")
	self:RegisterEvent("PLAYER_LEVEL_UP", "PLAYER_XP_UPDATE")
end

function PitBull_Experience:PLAYER_XP_UPDATE()
	self:UpdateExperience("player")
	if showPet then
		self:UpdateExperience("pet")
	end
end

function PitBull_Experience:UpdateExperience(unit, frame)
	if not unit then
		self:PLAYER_XP_UPDATE()
		return
	end
	if not frame then
		for frame in PitBull:IterateUnitFramesForUnit(unit) do
			self:UpdateExperience(unit, frame)
		end
		return
	end
	
	if self.db.profile[unit].ignore or UnitLevel(unit) == MAX_PLAYER_LEVEL or (unit == "pet" and UnitLevel("pet") == UnitLevel("player")) then
		if frame.expBar then
			PitBull_Experience:OnClearUnitFrame(unit, frame)
			PitBull:UpdateLayout(frame)
		end
		return
	else
		if not frame.expBar then
			PitBull_Experience:OnPopulateUnitFrame(unit, frame)
			PitBull:UpdateLayout(frame)
		end
	end
	
	local current, max
	if unit == "player" then
		current, max = UnitXP(unit), UnitXPMax(unit)
	else
		current, max = GetPetExperience()
	end
	if max == 0 then
		current = 0
		max = 1
	end
	local rest = unit == "player" and GetXPExhaustion() or 0
	if self.db.profile[unit].deficit ~= self.db.profile[unit].reverse then
		frame.expBar:SetValue(math.max(max - current - rest, 0) / max)
		frame.expBar.rest:SetValue((max - current) / max)
	else
		frame.expBar:SetValue(current / max)
		frame.expBar.rest:SetValue(math.min(current + rest, max) / max)
	end
	local r, g, b = unpack(self.db.profile.colors)
	local br, bg, bb = unpack(self.db.profile.bgColors)
	local rr, rg, rb = unpack(self.db.profile.restColors)
	if self.db.profile[unit].reverse then
		frame.expBar:SetStatusBarColor(br, bg, bb, 1)
		frame.expBar.bg:SetVertexColor(r, g, b, 1)
	else
		frame.expBar:SetStatusBarColor(r, g, b, 1)
		frame.expBar.bg:SetVertexColor(br, bg, bb, 1)
	end	
	frame.expBar.rest:SetStatusBarColor(rr, rg, rb, 1)
	frame.expBar.text:SetTextColor(unpack(self.db.profile.textColor))
end

function PitBull_Experience:OnUpdateFrame(unit, frame)
	if unit == "player" or (showPet and unit == "pet") then
		self:UpdateExperience(unit, frame)
	end
end

function PitBull_Experience:OnPopulateUnitFrame(unit, frame)
	if frame.expBar or self.db.profile[unit].ignore or UnitLevel(unit) == MAX_PLAYER_LEVEL then
		return 
	end
	if unit == "player" or (showPet and unit == "pet") then
		local expBar = newFrame("StatusBar", frame)
		frame.expBar = expBar
		expBar:SetMinMaxValues(0, 1)
		expBar:SetValue(0.5)
		expBar:SetStatusBarTexture(PitBull:GetStatusBarTexture())
		expBar:SetStatusBarColor(0, 0, 1, 1)
		
		local expBarRest = newFrame("StatusBar", expBar)
		frame.expBar.rest = expBarRest
		expBarRest:SetMinMaxValues(0, 1)
		expBarRest:SetValue(0.5)
		expBarRest:SetStatusBarTexture(PitBull:GetStatusBarTexture())
		expBarRest:SetStatusBarColor(1, 0, 1, 1)
		expBarRest:SetAllPoints(expBar)
		
		local a, b = expBar:GetFrameLevel(), expBarRest:GetFrameLevel()
		expBar:SetFrameLevel(b)
		expBarRest:SetFrameLevel(a)
		
		local expBarBG = newFrame("Texture", expBarRest, "BACKGROUND")
		frame.expBar.bg = expBarBG
		expBarBG:SetTexture(PitBull:GetStatusBarTexture())
		expBarBG:SetVertexColor(unpack(self.db.profile.bgColors))
		expBarBG:SetAllPoints(expBar)
		
		local font, fontsize = PitBull:GetFont()
		
		local expBarText = newFrame("FontString", expBar, "OVERLAY")
		frame.expBar.text = expBarText
		expBarText:SetFont(font, fontsize)
		expBarText:SetShadowColor(0, 0, 0, 1)
		expBarText:SetShadowOffset(0.8, -0.8)
		expBarText:SetNonSpaceWrap(false)
		expBarText:SetJustifyH("CENTER")
		expBarText:SetAllPoints(expBar)
		if unit == "player" then
			DogTag:RegisterFontString(frame.expBar.text, frame, unit, "[fractionalxp] [percentxp:percent:paren] [percentrestxp:hidezero:percent:prepend(R: )]")
		else
			DogTag:RegisterFontString(frame.expBar.text, frame, unit, "[fractionalxp] [percentxp:percent:paren]")
		end
	end
end

function PitBull_Experience:OnClearUnitFrame(unit, frame)
	if frame.expBar then
		frame.expBar.text = delFrame(frame.expBar.text)
		frame.expBar.bg = delFrame(frame.expBar.bg)
		frame.expBar.rest = delFrame(frame.expBar.rest)
		frame.expBar = delFrame(frame.expBar)
	end
end

function PitBull_Experience:OnUpdateStatusBarTexture(texture)
	for frame in PitBull:IterateUnitFramesForUnit("player") do
		if frame.expBar and frame.expBar.bg and frame.expBar.rest then
			frame.expBar:SetStatusBarTexture(texture)
			frame.expBar.bg:SetTexture(texture)
			frame.expBar.rest:SetStatusBarTexture(texture)
		end
	end
	if showPet then
		for frame in PitBull:IterateUnitFramesForUnit("pet") do
			if frame.expBar and frame.expBar.bg and frame.expBar.rest then
				frame.expBar:SetStatusBarTexture(texture)
				frame.expBar.bg:SetTexture(texture)
				frame.expBar.rest:SetStatusBarTexture(texture)
			end
		end
	end
end

function PitBull_Experience:OnUpdateFont(font, fontsize)
	for frame in PitBull:IterateUnitFramesForUnit("player") do
		if frame.expBar and frame.expBar.text then
			frame.expBar.text:SetFont(font, fontsize)
		end
	end
	if showPet then
		for frame in PitBull:IterateUnitFramesForUnit("pet") do
			if frame.expBar and frame.expBar.text then
				frame.expBar.text:SetFont(font, fontsize)
			end
		end
	end
end

function PitBull_Experience:OnUnknownLayout(unit, frame, name)
	frame.expBar:SetPoint("TOPLEFT", frame, "BOTTOMLEFT")
	frame.expBar:SetPoint("TOPRIGHT", frame, "BOTTOMRIGHT")
	frame.expBar:SetHeight(20)
end

local function getIgnored(group)
	return self.db.profile[group].ignore
end

local function setIgnored(group, value)
	self.db.profile[group].ignore = value
	PitBull_Experience:UpdateExperience(group)
end

local function getTextColor(group)
	return unpack(PitBull_Experience.db.profile.textColor)
end

local function setTextColor(group, r,g,b)
	PitBull_Experience.db.profile.textColor = {r, g, b}
	PitBull_Experience:UpdateExperience(group)
end

local function getDeficit(group)
	return PitBull_Experience.db.profile[group].deficit
end
local function setDeficit(group, value)
	PitBull_Experience.db.profile[group].deficit = value

	PitBull_Experience:UpdateExperience(group)
end
local function getReverse(group)
	return PitBull_Experience.db.profile[group].reverse
end
local function setReverse(group, value)
	PitBull_Experience.db.profile[group].reverse = value
	
	PitBull_Experience:UpdateExperience(group)
end


local function GetColor(group)
	return unpack(PitBull_Experience.db.profile.colors)
end

local function SetColor(group, r, g, b)
	PitBull_Experience.db.profile.colors = {r, g, b}
	
	PitBull_Experience:UpdateExperience(group)
end

local function GetBGColor(group)
	return unpack(PitBull_Experience.db.profile.bgColors)
end

local function SetBGColor(group, r, g, b)
	PitBull_Experience.db.profile.bgColors = {r, g, b}
	
	PitBull_Experience:UpdateExperience(group)
end

local function GetRestColor()
	return unpack(PitBull_Experience.db.profile.restColors)
end

local function SetRestColor(group, r, g, b)
	PitBull_Experience.db.profile.restColors = {r, g, b}
	
	PitBull_Experience:UpdateExperience(group)
end

PitBull.options.args.global.args.colors.args.experienceBar = {
	type = "group",
	name = L["Experience bar"],
	desc = L["Set the colors for the experience bar."],
	args = {
		text = {
			type = "color",
			name = L["Text color"],
			desc = L["Set the color of the text on the experience bar."],
			get = getTextColor,
			set = setTextColor,
		},
		bar = {
			type = "color",
			name = L["Color"],
			desc = L["Set the color for the experience bar."],
			get = GetColor,
			set = SetColor,
		},
		background = {
			type = "color",
			name = L["Background color"],
			desc = L["Set the background color for the experience bar."],
			get = GetBGColor,
			set = SetBGColor,
		},
		resting = {
			type = "color",
			name = L["Rest color"],
			desc = L["Set the color for rested experience on the bar."],
			get = GetRestColor,
			set = SetRestColor,
		},
	},
}

PitBull_Experience:RegisterPitBullOptionsMethod(function(group)
	if group == "player" or (showPet and group == "pet") then 
		return {
			name = L["Experience bar"],
			desc = L["Options for the units experience bar."],
			type = "group",
			args = {
				ignore = {
					type = 'toggle',
					name = L["Disable"],
					desc = L["Disable the experience bar."],
					get = getIgnored,
					set = setIgnored,
					passValue = group,
				},
				deficit = {
					type = "toggle",
					name = L["Show as deficit"],
					desc = L["Show experience deficit, draining the bar when you gain experience."],
					get = getDeficit,
					set = setDeficit,
					passValue = group,
					disabled = getIgnored,
				},
				reverse = {
					type = "toggle",
					name = L["Reverse"],
					desc = L["Reverses the direction of the experience bar, filling it from right to left."],
					get = getReverse,
					set = setReverse,
					passValue = group,
					disabled = getIgnored,
				},
			}
		}
	else
		return
	end
end)
